@@ -157,11 +157,11 @@ def doctest_load_options():
           -c, --config xxx       use this configuration file instead of the default
           -h, --help             show this help message
           -d, --daemon           go to background after starting
+          --pack                 pack the database
           -r, --restore-manager password
                                  restore the manager user with the provided password
                                  (read password from the standart input if 'password'
                                  is '-')
-          --manage               only do management tasks, don't run the server
         [exited with status 0]
 
     Here's what happens, when you use an unknown command line option.
